# Generated by Django 2.2.4 on 2019-08-05 20:01

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Issue',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=100, verbose_name='Name')),
                ('created_by',
                 models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='created_issues',
                                   to=settings.AUTH_USER_MODEL, verbose_name='Created by')),
                ('leader',
                 models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='leading_issue',
                                      to=settings.AUTH_USER_MODEL, verbose_name='Leader')),
                ('solver', models.OneToOneField(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE,
                                                related_name='solving_issue', to=settings.AUTH_USER_MODEL,
                                                verbose_name='Solver')),
                ('watched_by',
                 models.ManyToManyField(blank=True, related_name='watching_issues', to=settings.AUTH_USER_MODEL,
                                        verbose_name='Watched by')),
            ],
        ),
    ]
