# Generated by Django 3.1.7 on 2021-04-01 09:28

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        ('issue_tracker', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='issue',
            name='related_object_ct',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='contenttypes.contenttype'),
        ),
        migrations.AddField(
            model_name='issue',
            name='related_object_id',
            field=models.PositiveIntegerField(blank=True, null=True),
        ),
    ]
